#ifndef BOOK_H
#define BOOK_H

#include <qdom.h>
#include <qstring.h>
#include <qptrlist.h>

class Picture;

class Book 
{
    public:
        /* Constructor & Destructor */
        Book(const QString& inputFile);
        ~Book();

        /* Inspectors */
        QString getDir() const 
            { return dir_; }
        QString getFile() const 
            { return file_; }
        QString getAttribute(QString attr) const
            { return info_root_.attribute(attr); }
        int getNumAttribute(QString attr) const
            { return QString(info_root_.attribute(attr)).toInt(); }
        bool getBoolAttribute(QString attr) const
            { return (bool) QString(info_root_.attribute(attr)).toInt(); }
        QString html_getAttribute(QString attr) const
            { return info_html_.attribute(attr); }
        int html_getNumAttribute(QString attr) const
            { return QString(info_html_.attribute(attr)).toInt(); }
        bool html_getBoolAttribute(QString attr) const
            { return (bool) QString(info_html_.attribute(attr)).toInt(); }
        
        const QPtrList<Picture>* getPictures() const
            {  Q_CHECK_PTR(pictures_); return pictures_; }
        int getPictureIndex(const Picture* picture) const;
        bool containsPicture(const QString& fileName) const;
        
        /* Mutators */
        void setFile(const QString& file);
        void setAttribute(QString attr, QString val, bool overwrite = true) 
            { if(overwrite||!info_root_.hasAttribute(attr)) 
                info_root_.setAttribute(attr,val); }
        void setNumAttribute(QString attr, int val, bool overwrite = true)
            { if(overwrite||!info_root_.hasAttribute(attr))
                info_root_.setAttribute(attr,val); }
        void setBoolAttribute(QString attr, bool val, bool overwrite = true)
            { if(overwrite||!info_root_.hasAttribute(attr))
                info_root_.setAttribute(attr,(int) val); }
        void html_setAttribute(QString attr, QString val, bool overwrite = true) 
            { if(overwrite||!info_html_.hasAttribute(attr)) 
                info_html_.setAttribute(attr,val); }
        void html_setNumAttribute(QString attr, int val, bool overwrite = true)
            { if(overwrite||!info_html_.hasAttribute(attr))
                info_html_.setAttribute(attr,val); }
        void html_setBoolAttribute(QString attr, bool val, bool overwrite = true)
            { if(overwrite||!info_html_.hasAttribute(attr))
                info_html_.setAttribute(attr,(int) val); }
        void initAttributes();
        
        void addPicture(Picture* picture);
        void removePicture(Picture* picture);
        void movePictureForward(Picture* picture);
        void movePictureBackward(Picture* picture);

        /* I/O methods */
        void readFromFile(const QString& file);
        void writeToFile() const;
        
    protected:
        void writeToFile(const QString& file) const;
        QPtrList<Picture>* pictures() const
            { Q_CHECK_PTR(pictures_) ; return pictures_; }
    
    private:
        QString file_, dir_;
        QPtrList<Picture>* pictures_;
        QDomElement info_root_; 
        QDomElement info_html_; 
};

#endif
