#include <qdir.h>
#include <qmessagebox.h>
#include <qobject.h>
#include <qptrlist.h>
#include <qstring.h>
#include <libxslt/xslt.h>
#include <libxslt/transform.h>
#include <libexslt/exslt.h>

#include "book.h"
#include "htmlgenerator.h"
#include "main.h"
#include "picture.h"

extern int xmlLoadExtDtdDefaultValue;

void copyStylesheet(const QString& sourceFile, const QString& targetFile);

void createHTML(const Book* book)
{
    /* Initialize target dirs */
    QString relRootDir;
    QString htmlTarget(book->getAttribute("html_dir"));
    if (!(htmlTarget.isEmpty() || htmlTarget == ".")) 
        relRootDir = "../";
    QString thumbsDir = relRootDir + book->getAttribute("thumbs_dir");
    QString originalsDir = relRootDir + book->getAttribute("originals_dir");
    QDir targetDir(book->getDir());
    if (!(book->getAttribute("html_dir").isEmpty()
            || targetDir.exists(book->getAttribute("html_dir"))
                || targetDir.mkdir(book->getAttribute("html_dir")))) {
	    QMessageBox::critical(0, QObject::tr("Error"), 
                                 QObject::tr("Error creating HTML dir"),
	    QMessageBox::Ok,QMessageBox::NoButton);
	    return;
    }
    targetDir.cd(book->getAttribute("html_dir"));

    /* Save index file */
    book->writeToFile();

    /* Create HTML */
    xsltStylesheetPtr stylesheet;
    xmlDocPtr doc, res;
    xsltTransformContextPtr context;

    xmlSubstituteEntitiesDefault(1);
    xmlLoadExtDtdDefaultValue = 1;
    exsltCommonRegister();
    stylesheet = xsltParseStylesheetFile((const xmlChar *) 
            book->html_getAttribute("template").latin1());
    if (stylesheet == NULL) {
        QMessageBox::critical(0, QObject::tr("Error"), 
            QString(QObject::tr("Error parsing '%1'")).
                arg(book->html_getAttribute("template")),
            QMessageBox::Ok,QMessageBox::NoButton);
        return;
    }
    doc = xmlParseFile(QDir(book->getDir()).filePath(book->getFile()).latin1());
    if (doc == NULL) {
        QMessageBox::critical(0, QObject::tr("Error"), 
            QString(QObject::tr("Error parsing '%1'")).
                arg(QDir(book->getDir()).filePath(book->getFile())),
            QMessageBox::Ok,QMessageBox::NoButton);
        return;
    }
    context = xsltNewTransformContext(stylesheet, doc);
    res = xsltApplyStylesheetUser(stylesheet, doc, NULL,
            QString("%1/").arg(targetDir.path()).latin1(),NULL,context);
    if (res == NULL) {
        QMessageBox::critical(0, QObject::tr("Error"), 
            QString(QObject::tr("Error applying XSLT template")),
            QMessageBox::Ok,QMessageBox::NoButton);
        return;
    }
    
    xsltFreeStylesheet(stylesheet);
    xmlFreeDoc(res);
    xmlFreeDoc(doc);
    xsltCleanupGlobals();
    xmlCleanupParser();

    /* Copy stylesheet */
    copyStylesheet(book->html_getAttribute("stylesheet_file"),
        targetDir.filePath(book->html_getAttribute("stylesheet")));
}


void copyStylesheet(const QString& sourceFile, const QString& targetFile)
{
    /* Check if the file has to be copied */
    if (QDir(QFileInfo(sourceFile).dirPath()).canonicalPath() !=
        QDir(QFileInfo(targetFile).dirPath()).canonicalPath()) {
        /* Open the source & target files */
        QFile source_file(sourceFile);
        QFile target_file(targetFile);
        if (source_file.open(IO_ReadOnly) 
            && target_file.open(IO_WriteOnly)) {
            /* Copy all the data */
            char data[4096];
            while (!source_file.atEnd()) {
                Q_LONG read = source_file.readBlock(data,4096);
                target_file.writeBlock(data,read);
            }
        }
        else 
            qWarning(QString("Error copying %1 to %2").arg(sourceFile).
                                                       arg(targetFile));
    }
}
