#ifndef IMAGEMANAGER_H
#define IMAGEMANAGER_H

#include <qimage.h>
#include <qpixmap.h>
#include <qstring.h>

class Picture;

class ImageManager
{
    friend class ImageScaleThread; 
    friend class ImageRotateThread;

    public:
        static QPixmap getThumb(const Picture* picture);
        static void importPicture(const Picture* picture, const QString& file);
        static void rotatePicture(const Picture* picture, double deg);
        static void refresh(const Picture* picture);
        static bool isReadable(const QString& file);
        static bool isDirty(const QString& oldFile, const QString& newFile);
        static void cleanup(const Picture* picture, bool cleanupFile = false);

        static QString supportedImageFormats;

    protected:
        static void rotate(const QString& image, double deg);
        static void scaleImage(const QString& source, const QString& target,
                               int w, int h, int q, QImage::ScaleMode mode);
        static void scale(const QString& source, const QString& target,
                          int w, int h, int quality, QImage::ScaleMode mode);
        static void copyImage(const QString& source, const QString& target);
        //static void copy(const QString& source, const QString& target);
        static void rebuildThumb(const Picture* picture);
        static void rebuildSlide(const Picture* picture);
};

#endif
