#include <qdir.h>
#include <qfileinfo.h>
#include <qimage.h>
#include <qstring.h>

#include "book.h"
#include "imagemanager.h"
#include "picture.h"


Picture::Picture(Book* book, const QString& name, const QString& description) : 
        book_(book), fileName_(name), description_(description)
{
    Q_CHECK_PTR(book);
    book_->addPicture(this);
}

int Picture::getIndex() const
{
    return book()->getPictureIndex(this);
}


void Picture::moveBackward()
{
    book()->movePictureBackward(this);
}

void Picture::moveForward()
{
    book()->movePictureForward(this);
}

bool Picture::rename(const QString& name)
{
    if (book()->containsPicture(name)) 
        return false;
    
    QDir dir = QDir(QFileInfo(filePath()).dirPath());
    qDebug(QString("Renaming %1 to %2").arg(dir.filePath(fileName()))
                                       .arg(dir.filePath(name)));
    if (!dir.rename(fileName(),name))
        return false;

    ImageManager::cleanup(this);
    fileName_ = name;
    ImageManager::refresh(this);

    return true;
}

void Picture::remove(bool removeFile)
{
    ImageManager::cleanup(this,removeFile);
    book()->removePicture(this);
}


QString Picture::filePath() const
{
    return QString("%1/%2/%3").arg(book()->getDir())
                              .arg(book()->getAttribute("originals_dir"))
                              .arg(fileName());
} 

QString Picture::thumbPath() const
{
    return QString("%1/%2/%3").arg(book()->getDir())
                              .arg(book()->getAttribute("thumbs_dir"))
                              .arg(fileName());
}

QString Picture::slidePath() const
{
    return QString("%1/%2/%3").arg(book()->getDir())
                              .arg(book()->getAttribute("slides_dir"))
                              .arg(fileName());
} 

int Picture::thumbWidth() const 
    { return book()->getNumAttribute("thumbs_width"); }
    
int Picture::thumbHeight() const
    { return book()->getNumAttribute("thumbs_height"); }

int Picture::thumbQuality() const
    { return book()->getNumAttribute("thumbs_quality"); }
    
QImage::ScaleMode Picture::thumbScaleMode() const
    { return (QImage::ScaleMode) book()->getNumAttribute("thumbs_scalemode"); }


bool Picture::hasSlide() const
    { return book()->getBoolAttribute("slides_create"); }

int Picture::slideWidth() const
    { return book()->getNumAttribute("slides_width"); }
    
int Picture::slideHeight() const
    { return book()->getNumAttribute("slides_height"); }

int Picture::slideQuality() const
    { return book()->getNumAttribute("slides_quality"); }

QImage::ScaleMode Picture::slideScaleMode() const
    { return (QImage::ScaleMode) book()->getNumAttribute("slides_scalemode"); }

int Picture::originalWidth() const
    { return book()->getNumAttribute("originals_width"); }

int Picture::originalHeight() const
    { return book()->getNumAttribute("originals_height"); }

int Picture::originalQuality() const
    { return book()->getNumAttribute("originals_quality"); }

QImage::ScaleMode Picture::originalScaleMode() const
    { return (QImage::ScaleMode) 
            book()->getNumAttribute("originals_scalemode"); }
