#ifndef PICTURE_H
#define PICTURE_H

#include <qimage.h>
#include <qstring.h>

class Book;

class Picture
{
    public:
        Picture(Book* book, const QString& name, const QString& description);
       
        void setDescription(const QString& description)
            { description_ = description; }
        int getIndex() const;
        void moveBackward();
        void moveForward();
        bool rename(const QString& name);
        void remove(bool removeFile = false);
        
        QString description() const { return description_; }
        
        int thumbWidth() const;
        int thumbHeight() const;
        int thumbQuality() const;
        QImage::ScaleMode thumbScaleMode() const;
                
        bool hasSlide() const;
        int slideWidth() const;
        int slideHeight() const;
        int slideQuality() const;
        QImage::ScaleMode slideScaleMode() const;
                
        int originalWidth() const;
        int originalHeight() const;
        int originalQuality() const;
        QImage::ScaleMode originalScaleMode() const;
        
        QString fileName() const { return fileName_; }
        QString filePath() const;
        QString thumbPath() const;
        QString slidePath() const;
        Book* getBook() const { Q_CHECK_PTR(book_); return book_;}
        
    protected:
        Book* book() const { Q_CHECK_PTR(book_); return book_; }

    private:
        Book* book_;
        QString fileName_;
        QString description_;
};

#endif
