#include <qpopupmenu.h>
#include <qiconset.h>
#include <qiconview.h>
#include <qinputdialog.h>
#include <qimage.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qmime.h>
#include <qpixmap.h>

#include "picture.h"
#include "picturecontextmenu.h"
#include "pictureitem.h"

#include "indexwindow.h"


QPixmap PictureContextMenu::loadPixmap(const QString& name)
{
    const QMimeSource *m = QMimeSourceFactory::defaultFactory()->data( name );
    if ( !m )
        return QPixmap();
    QPixmap pix;
    QImageDrag::decode( m, pix );
    return pix;
}


PictureContextMenu::PictureContextMenu(QWidget * parent, 
        PictureItem* picture) : QPopupMenu(parent), picture_(picture)
{
    insertItem(QIconSet(loadPixmap("gimp.png")),"Edit",this,SLOT(edit()));
#ifdef HAVE_IMAGEMAGICKPP
    rotateMenu_ = new QPopupMenu(this);
    rotateMenu_->insertItem("90 clockwise",this,SLOT(rotate90()));
    rotateMenu_->insertItem("180",this,SLOT(rotate180()));
    rotateMenu_->insertItem("90 counterclockwise",this,SLOT(rotate270()));
    insertItem("Rotate",rotateMenu_);
#endif
    insertItem(QIconSet(loadPixmap("refresh.png")),
            "Refresh",this,SLOT(refresh()));
    insertItem(QIconSet(loadPixmap("rebuild.png")),
            "Rebuild",this,SLOT(rebuild()));
    insertItem("Rename",this,SLOT(rename()));
    insertItem(QIconSet(loadPixmap("edittrash.png")),
            "Remove",this,SLOT(remove()));
//    insertItem("Info",this,SLOT(info()));
}


PictureContextMenu::~PictureContextMenu()
{
#ifdef HAVE_IMAGEMAGICKPP
    delete rotateMenu_;
#endif
}


void PictureContextMenu::edit()
{
    picture()->edit();
}

void PictureContextMenu::info()
{
}

void PictureContextMenu::refresh()
{
    picture()->refresh();
}

void PictureContextMenu::rebuild()
{
    picture()->rebuild();
}

void PictureContextMenu::rename()
{
    QString old = picture()->getPicture()->fileName();

    bool ok=false;
    QString target = QInputDialog::getText(
        "Spit", "Rename '"+ old +"' to:",
        QLineEdit::Normal,QString::null, &ok, this );

    if (ok) 
        picture()->rename(target);
}


void PictureContextMenu::remove()
{
    switch(QMessageBox::information(this, "Removing picture",
            "Do you want to remove the file from disk ?\n",
            QMessageBox::Yes, QMessageBox::No | QMessageBox::Default,
            QMessageBox::Cancel | QMessageBox::Escape)) {
        case QMessageBox::Yes : picture()->remove(true); break;
        case QMessageBox::No : picture()->remove(false); break;
        case QMessageBox::Cancel : return;
    }
    QIconView* view = picture()->iconView();
    delete picture();
    view->sort();
}


void PictureContextMenu::rotate90()
{
    picture()->rotate(90);
}

void PictureContextMenu::rotate180()
{   
    picture()->rotate(180);
}

void PictureContextMenu::rotate270()
{   
    picture()->rotate(270);
}
