#include <qiconview.h>
#include <qmessagebox.h>
#include <qpixmap.h>
#include <qpopupmenu.h>
#include <qstring.h>


#include "editor.h"
#include "picture.h"
#include "pictureitem.h"
#include "indexwindow.h"
#include "imagemanager.h"

PictureItem::PictureItem(QIconView*  parent, Picture* picture) :
    QIconViewItem(parent), picture_(picture)
{
    Q_CHECK_PTR(parent); 
    Q_CHECK_PTR(picture); 
    
    setRenameEnabled(true);
    setDragEnabled(false);
    setDropEnabled(false);
    setText(getPicture()->description());
    setPixmap(QPixmap());
    setPixmap(ImageManager::getThumb(getPicture()));
}
            

int PictureItem::compare(QIconViewItem* i) const
{
    Q_CHECK_PTR(i);
    
    PictureItem* pic = (PictureItem*) i;
    return (pic->getPicture()->getIndex() > getPicture()->getIndex() ? -1 
            : (pic->getPicture()->getIndex() < getPicture()->getIndex() ? 1 : 0)); 
}


void PictureItem::setDescription(const QString& text)
{
    getPicture()->setDescription(text);
}


void PictureItem::moveBackward()
{
    getPicture()->moveBackward();
}


void PictureItem::moveForward()
{
    getPicture()->moveForward();
}


void PictureItem::edit()
{
    Editor::edit(this);
}


void PictureItem::refresh()
{
    ImageManager::refresh(getPicture());
    setPixmap(ImageManager::getThumb(getPicture()));
    iconView()->sort();
}


void PictureItem::rebuild()
{
    ImageManager::cleanup(getPicture());
    ImageManager::refresh(getPicture());
    setPixmap(ImageManager::getThumb(getPicture()));
    iconView()->sort();
}


void PictureItem::remove(bool removeFile)
{
    getPicture()->remove(removeFile);
}


void PictureItem::rename(QString& name)
{
    if (!getPicture()->rename(name)) {
         QMessageBox::warning( iconView(), "Spit",
            QString("Could not rename '%1' to '%2'").
                arg(getPicture()->fileName()).arg(name),
            QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton);
    }
}

void PictureItem::rotate(double degrees)
{
    ImageManager::rotatePicture(getPicture(),degrees);
    refresh();
}
