#include <qspinbox.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qcolor.h>
#include <qcolordialog.h>
#include <qfiledialog.h>
#include <qfileinfo.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <qimage.h>

#ifdef HAVE_CONFIG
#include "config.h"
#else
#include "defaults.h"
#endif

#include "settingsdialog.h"
#include "book.h"


QImage::ScaleMode indexToScaleMode(int i)
{
    switch (i) {
        case 0 : return QImage::ScaleMin;
        case 1 : return QImage::ScaleMax;
        case 2 : return QImage::ScaleFree;
        default : return QImage::ScaleMin;
    }
}


int scaleModeToIndex(QImage::ScaleMode i) 
{
    switch (i) {
        case QImage::ScaleMin : return 0;
        case QImage::ScaleMax : return 1;
        case QImage::ScaleFree : return 2;
        default : return 0;
    }
}


void SettingsDialog::exec()
{
    /* Initialize thumbs settings */
    text_thumbsWidth->setValue(book()->getNumAttribute("thumbs_width"));
    text_thumbsHeight->setValue(book()->getNumAttribute("thumbs_height"));
    text_thumbsQuality->setValue(book()->getNumAttribute("thumbs_quality"));
    combo_thumbsScaleMode->setCurrentItem(scaleModeToIndex(
            (QImage::ScaleMode) book()->getNumAttribute("thumbs_scalemode")));
    text_thumbsPerLine->setValue(book()->html_getNumAttribute("thumbs_per_line"));
    text_linesPerPage->setValue(book()->html_getNumAttribute("lines_per_page"));

    /* Initialize slideshow settings */
    text_slidesWidth->setValue(book()->getNumAttribute("slides_width"));
    text_slidesHeight->setValue(book()->getNumAttribute("slides_height"));
    text_slidesQuality->setValue(book()->getNumAttribute("slides_quality"));
    combo_slidesScaleMode->setCurrentItem(scaleModeToIndex(
            (QImage::ScaleMode) book()->getNumAttribute("slides_scalemode")));
    text_slidesWidth->setEnabled(book()->getBoolAttribute("slides_create"));
    text_slidesHeight->setEnabled(book()->getBoolAttribute("slides_create"));
    text_slidesQuality->setEnabled(book()->getBoolAttribute("slides_create"));
    combo_slidesScaleMode->setEnabled(book()->getBoolAttribute("slides_create"));
    check_createSlides->setChecked(book()->getBoolAttribute("slides_create"));
                
    /* Initialize originals settings */
    text_originalsWidth->setValue(book()->getNumAttribute("originals_width"));
    text_originalsHeight->setValue(book()->getNumAttribute("originals_height"));
    text_originalsQuality->setValue(
        book()->getNumAttribute("originals_quality"));
    combo_originalsScaleMode->setCurrentItem( scaleModeToIndex(
        (QImage::ScaleMode) book()->getNumAttribute("originals_scalemode")));
    check_originalsScale->setChecked(
        book()->getBoolAttribute("originals_scale"));
    //check_originalsScale->setEnabled(
    //    book()->getBoolAttribute("originals_create"));
    check_originalsScale->setEnabled(false); // FIXME: temporary
    check_originalsCreate->setChecked(
        book()->getBoolAttribute("originals_create"));
    //text_originalsWidth->setEnabled(
    //    book()->getBoolAttribute("originals_scale")
    //    && book()->getBoolAttribute("originals_create"));
    text_originalsWidth->setEnabled(false);
    //text_originalsHeight->setEnabled(
     //   book()->getBoolAttribute("originals_scale")
     //   && book()->getBoolAttribute("originals_create"));
    text_originalsHeight->setEnabled(false);
    //text_originalsQuality->setEnabled(
    //    book()->getBoolAttribute("originals_scale")
    //    && book()->getBoolAttribute("originals_create"));
    text_originalsQuality->setEnabled(false);
    //combo_originalsScaleMode->setEnabled(
    //    book()->getBoolAttribute("originals_scale") 
    //    && book()->getBoolAttribute("originals_create"));
    combo_originalsScaleMode->setEnabled(false);
    //text_originalsScale->setEnabled(
    //    book()->getBoolAttribute("originals_create"));
    text_originalsScale->setEnabled(false);
    
   
    /* Initialize HTML settings */
    text_template->setText(book()->html_getAttribute("template"));
    text_stylesheet->setText(book()->html_getAttribute("stylesheet_file"));
    text_linkup->setText(book()->html_getAttribute("linkup"));
    
    /* Initialize files settings */
    /*text_originalsDir->setText(book()->getAttribute("originals_dir"));
    text_thumbsDir->setText(book()->getAttribute("thumbs_dir"));
    text_slidesDir->setText(book()->getAttribute("slides_dir"));
    text_htmlDir->setText(book()->getAttribute("html_dir"));*/
    
    /* Initialize misc. settings */
    /*check_descDefault->setChecked(book()->getBoolAttribute("desc_default"));
    text_editor->setText(book()->getAttribute("editor"));
    text_browser->setText(book()->getAttribute("browser"));*/

    /* Fix size. FIXME: shouldn't this be cleaner ? */
    setMinimumWidth(width());
    setMaximumWidth(width());
    setMinimumHeight(height());
    setMaximumHeight(height());

    /* Do other stuff */
    QDialog::exec();
}

void SettingsDialog::accept()
{
    /* Perform some checks */
    /*if (text_thumbsDir->text().contains("..") 
        || text_thumbsDir->text().contains("/")
        || text_slidesDir->text().contains("..")
        || text_slidesDir->text().contains("/")
        || text_originalsDir->text().contains("/")
        || text_originalsDir->text().contains("..")
        || text_htmlDir->text().contains("/")
        || text_htmlDir->text().contains("..")) {
        QMessageBox::warning( this, "Spit",
                "Directory names cannot contain '..' or '/'",
                QMessageBox::Ok | QMessageBox::Default,
                QMessageBox::NoButton);
        return;
    }*/

    /* Save thumbs settings */
    book()->setNumAttribute("thumbs_width",text_thumbsWidth->value()); 
    book()->setNumAttribute("thumbs_height",text_thumbsHeight->value());
    book()->setNumAttribute("thumbs_quality",text_thumbsQuality->value());
    book()->setNumAttribute("thumbs_scalemode",
        indexToScaleMode(combo_thumbsScaleMode->currentItem()));
    book()->html_setNumAttribute("thumbs_per_line",text_thumbsPerLine->value());
    book()->html_setNumAttribute("lines_per_page",text_linesPerPage->value());

    /* Save slideshow settings */
    book()->setNumAttribute("slides_width",text_slidesWidth->value());
    book()->setNumAttribute("slides_height",text_slidesHeight->value());
    book()->setNumAttribute("slides_scalemode",
        indexToScaleMode(combo_slidesScaleMode->currentItem()));
    book()->setNumAttribute("slides_quality",text_slidesQuality->value());
    book()->setBoolAttribute("slides_create",check_createSlides->isChecked());
                 
    /* Save originals settings */
    book()->setNumAttribute("originals_width",text_originalsWidth->value());
    book()->setNumAttribute("originals_height",text_originalsHeight->value());
    book()->setNumAttribute("originals_quality",text_originalsQuality->value());
    book()->setNumAttribute("originals_scalemode",indexToScaleMode(
        combo_originalsScaleMode->currentItem()));
    book()->setBoolAttribute("originals_scale",
        check_originalsScale->isChecked());
    book()->setBoolAttribute("originals_create",
        check_originalsCreate->isChecked());

    /* Save HTML settings */
    book()->html_setAttribute("stylesheet_file",text_stylesheet->text());
    book()->html_setAttribute("stylesheet",
            QFileInfo(text_stylesheet->text()).fileName());
    book()->html_setAttribute("template",text_template->text());
    book()->html_setAttribute("linkup",text_linkup->text());

    /* Save file settings */
    /*book()->setAttribute("originals_dir",text_originalsDir->text());
    book()->setAttribute("thumbs_dir",text_thumbsDir->text());
    book()->setAttribute("slides_dir",text_slidesDir->text());
    book()->setAttribute("html_dir",text_htmlDir->text());*/
    
    /* Save misc. settings */
    /*book()->setBoolAttribute("desc_default",check_descDefault->isChecked());
    book()->setAttribute("editor",text_editor->text());
    book()->setAttribute("browser",text_browser->text());*/

    /* Do other stuff */
    QDialog::accept();
};


void SettingsDialog::button_template_clicked()
{
    QString file = QFileDialog::getOpenFileName( LIBDIR "templates/",
            "Templates (*.xsl)", this, "open template dialog", 
            tr("Open Template ..."));
    if (!file.isEmpty()) 
        text_template->setText(file);
}


void SettingsDialog::button_stylesheet_clicked()
{
    QString file = QFileDialog::getOpenFileName( LIBDIR "templates/",
            "Stylesheets (*.css)", this, "open stylesheet dialog", 
            tr("Open Stylesheet ..."));
    if (!file.isEmpty()) 
        text_stylesheet->setText(file);
}


void SettingsDialog::check_originalsCreate_toggled(bool /*toggled*/)
{
    // FIXME: temporarily out of order
    /*text_originalsWidth->setEnabled(
            check_originalsScale->isChecked() && toggled);
    text_originalsHeight->setEnabled(
            check_originalsScale->isChecked() && toggled);
    text_originalsQuality->setEnabled(
            check_originalsScale->isChecked() && toggled);
    text_originalsScale->setEnabled(toggled);
    combo_originalsScaleMode->setEnabled(
            check_originalsScale->isChecked() && toggled);*/
}
